"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
exports.logout = exports.loadTokens = exports.refreshTokens = exports.getAuthenticationURL = exports.shouldRefreshTokens = exports.getCurrentUser = exports.onOauthChange = exports.currentUser = void 0;
var axios_1 = require("axios");
var jwt_decode_1 = require("jwt-decode");
var url = require("url");
var qs = require("qs");
var keytar = require("keytar");
var config_json_1 = require("../config.json");
var REFRESH_TRESHOLD = 30000;
var APP_SERVICE = 'kloke-app';
exports.currentUser = null;
var callbacks = [];
var onOauthChange = function (callback) {
    callbacks.push(callback);
};
exports.onOauthChange = onOauthChange;
function executeCallbacks() {
    callbacks.forEach(function (callback) { return callback(); });
}
function getCurrentUser() {
    return exports.currentUser;
}
exports.getCurrentUser = getCurrentUser;
function shouldRefreshTokens() {
    if (exports.currentUser === null) {
        return false;
    }
    else {
        var expirationDate = exports.currentUser.expirationDate;
        var expirationDateWithTreshold = new Date(expirationDate.getTime() - REFRESH_TRESHOLD);
        return expirationDateWithTreshold && expirationDateWithTreshold < new Date();
    }
}
exports.shouldRefreshTokens = shouldRefreshTokens;
function getAuthenticationURL() {
    var queryParams = {
        client_id: config_json_1.CLIENT_ID,
        redirect_uri: config_json_1.REDIRECT_URI,
        scope: 'local_tokens',
        response_type: 'code'
    };
    return config_json_1.AUTH0_DOMAIN + "/oauth?" + qs.stringify(queryParams);
}
exports.getAuthenticationURL = getAuthenticationURL;
function fetchAuthData(body) {
    return __awaiter(this, void 0, void 0, function () {
        var options, response, data, error_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    options = {
                        headers: {
                            Authorization: "Basic " + config_json_1.HEADER_AUTHORIZATION
                        }
                    };
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 5]);
                    return [4 /*yield*/, axios_1["default"].post(config_json_1.AUTH0_DOMAIN + "/oauth/v2/token", qs.stringify(body), options)];
                case 2:
                    response = _a.sent();
                    data = response.data;
                    setUserData(data);
                    executeCallbacks();
                    return [3 /*break*/, 5];
                case 3:
                    error_1 = _a.sent();
                    return [4 /*yield*/, logout()];
                case 4:
                    _a.sent();
                    throw error_1;
                case 5: return [2 /*return*/];
            }
        });
    });
}
function setUserData(data) {
    var access_token = data.access_token, expires_in = data.expires_in, refresh_token = data.refresh_token;
    var phone_number = jwt_decode_1["default"](access_token).phone_number;
    exports.currentUser = {
        accessToken: access_token,
        expirationDate: new Date(new Date().getTime() + expires_in * 1000),
        phoneNumber: phone_number
    };
    if (refresh_token) {
        keytar.setPassword(APP_SERVICE, 'account', refresh_token);
    }
}
function refreshTokens() {
    return __awaiter(this, void 0, void 0, function () {
        var refreshToken, body;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, keytar.getPassword(APP_SERVICE, 'account')];
                case 1:
                    refreshToken = _a.sent();
                    if (!exports.currentUser) {
                        throw new Error('No available refresh token.');
                    }
                    if (refreshToken) {
                        body = {
                            grant_type: 'refresh_token',
                            refresh_token: refreshToken
                        };
                        fetchAuthData(body);
                    }
                    return [2 /*return*/];
            }
        });
    });
}
exports.refreshTokens = refreshTokens;
function loadTokens(callbackURL) {
    return __awaiter(this, void 0, void 0, function () {
        var urlParts, query, body;
        return __generator(this, function (_a) {
            urlParts = url.parse(callbackURL, true);
            query = urlParts.query;
            body = {
                grant_type: 'authorization_code',
                code: "" + query.code,
                redirect_uri: config_json_1.REDIRECT_URI
            };
            fetchAuthData(body);
            return [2 /*return*/];
        });
    });
}
exports.loadTokens = loadTokens;
function logout() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, keytar.deletePassword(APP_SERVICE, 'accoount')];
                case 1:
                    _a.sent();
                    exports.currentUser = null;
                    executeCallbacks();
                    return [2 /*return*/];
            }
        });
    });
}
exports.logout = logout;
//# sourceMappingURL=oauth2-manager.js.map