import { contextBridge, ipcRenderer } from 'electron'
import {
  AuthDetails,
  EditKlinkDetails,
  KlinkDetails,
  ViewKlinkDetails,
} from '../common/types'

// Text is saved on preload script execution to prevent "clipboard monitoring"
const text = ipcRenderer.sendSync('get-clipboard-text')

const target = ipcRenderer.sendSync('get-target')

const isDev = ipcRenderer.sendSync('is-dev')

contextBridge.exposeInMainWorld('api', {
  isDev: () => isDev,
  getClipboardText: () => text,
  writeClipboardText: (text: string) => ipcRenderer.invoke('write-clipboard-text', text),
  close: () => ipcRenderer.invoke('close-window'),
  getAuthDetails: () => ipcRenderer.sendSync('get-auth-details'),
  login: async (phone: string, password: string) => {
    ipcRenderer.send('login', phone, password)

    return new Promise<AuthDetails>((resolve, reject) => {
      ipcRenderer.once('login', (event, result) => {
        if (result && result.error) {
          return reject(result.message)
        }

        resolve(result)
      })
    })
  },
  loginWithKloke: () => ipcRenderer.send('login-with-kloke'),
  confirmTfa: async (code: string) => {
    ipcRenderer.send('confirm-tfa', code)

    return new Promise<void>((resolve, reject) => {
      ipcRenderer.once('confirm-tfa', (event, result) => {
        if (result && result.error) {
          return reject(result.message)
        }

        resolve()
      })
    })
  },
  createKlink: (klinkRequest: KlinkDetails) => {
    ipcRenderer.send('create-klink', klinkRequest)

    return new Promise<string>((resolve, reject) => {
      ipcRenderer.once('create-klink', (event, result) => {
        if (result && result.error) {
          return reject(result.message)
        }

        resolve(result)
      })
    })
  },
  editKlink: async (token: string, klinkRequest: KlinkDetails) => {
    ipcRenderer.send('edit-klink', token, klinkRequest)

    return new Promise<void>((resolve, reject) => {
      ipcRenderer.once('edit-klink', (event, result) => {
        if (result && result.error) {
          return reject(result.message)
        }

        resolve()
      })
    })
  },
  readKlink: (token: string) => {
    ipcRenderer.send('read-klink', token)

    return new Promise<EditKlinkDetails | ViewKlinkDetails>((resolve, reject) => {
      ipcRenderer.once('read-klink', (event, result) => {
        if (result && result.error) {
          return reject(result.message)
        }

        resolve(result)
      })
    })
  },
  getCurrentUserKlinks: (page?: number) => {
    ipcRenderer.send('get-current-user-klinks', page)

    return new Promise<EditKlinkDetails[]>((resolve, reject) => {
      ipcRenderer.once('get-current-user-klinks', (event, result) => {
        if (result && result.error) {
          return reject(result.message)
        }

        resolve(result)
      })
    })
  },
  generateKlink: (token: string) => ipcRenderer.sendSync('generate-klink', token),
  getTarget: () => target,
})
