import * as Pushy from 'pushy-electron'
import * as config from '../config.json'
import log from 'electron-log'
// import * as qs from 'qs'
import { createClient } from './api-client'
import { getCurrentUser } from './oauth2-manager'
import { onOauthChange } from './oauth2-manager'

type Message = {
  title: string
  reader: string
  token: string
}

const client = createClient()

export async function registerPushy(callback: (message: Message) => void) {
  log.info('[Push] Registering for push notifications')

  Pushy.listen()
  Pushy.setNotificationListener(callback)

  const deviceToken = await Pushy.register({ appId: config.PUSHY_APP_ID })

  onOauthChange(async () => {
    await saveDeviceToken(deviceToken)
  })

  await saveDeviceToken(deviceToken)
}

export async function saveDeviceToken(deviceToken: string): Promise<void> {
  const user = getCurrentUser()

  if (user !== null) {
    try {
      await client.post('/frontvault/v1/devices', {
        device_id: deviceToken,
      })
    } catch (e) {
      log.error(`[Push] Unable to save device token: ${e.message}`)
    }
  }
}

// export async function deleteDeviceToken(deviceToken: string): Promise<void> {
//   const user = getCurrentUser()

//   if (user !== null) {
//     try {
//       await client.delete('/frontvault/v1/devices', qs.stringify({
//         device_id: deviceToken,
//       }))
//     } catch (e) {
//       log.error(`[Push] Unable to save device token: ${e.message}`)
//     }
//   }
// }
