import React from 'react'
import { HashRouter as Router, Redirect, Route, Switch } from 'react-router-dom'
import { ThemeProvider } from '@material-ui/styles'
import CreateKlinkDialog from './features/CreateKlinkDialog'
import CopyKlinkDialog from './features/CopyKlinkDialog'
import AuthDialog from './features/AuthDialog'
import ConfirmTfaDialog from './features/ConfirmTfaDialog'
import KlinkDetailsDialog from './features/KlinkDetailsDialog'
import DeklinkDialog from './features/DeklinkDialog'
import Klinks from './features/Klinks'
import { mainTheme, AppContainer, Banner } from './styles'
import banner from './pics/Klink_Banner.png'

const unprotectedRoutes = [
  '/login',
  '/login/tfa',
]

const App = () => {
  return (
    <Router>
      <ThemeProvider theme={mainTheme}>
        <Banner>
          <img src={banner} alt="banner"></img>
        </Banner>
        <AppContainer>
          <Switch>
            <Route
              path="/"
              exact
              render={({ match }) => {
                const target = window.api.getTarget()
                const auth = window.api.getAuthDetails()

                if ((auth === null) && !unprotectedRoutes.includes(match.path)) {
                  return (
                    <Redirect to="/login" />
                  )
                }

                if (target === '/') {
                  return null
                }

                if (target === null) {
                  return (
                    <Redirect to="/create-klink" />
                  )
                }

                return (
                  <Redirect to={target} />
                )
              }}
            />
            <Route
              path="/login"
              exact
              render={({ history }) => {
                const auth = window.api.getAuthDetails()

                if (auth !== null && auth.tfaMissing) {
                  return (
                    <Redirect to="/login/tfa" />
                  )
                }

                return <AuthDialog />
              }}
            />
            <Route
              path="/login/tfa"
              exact
              render={({ history }) => {
                const auth = window.api.getAuthDetails()

                if (auth === null) {
                  return (
                    <Redirect to="/login" />
                  )
                }

                return (
                  <ConfirmTfaDialog
                    confirmTfa={async (code) => {
                      await window.api.confirmTfa(code)

                      if (window.api.getTarget() === '/login') {
                        window.api.close()

                        return
                      }

                      history.replace('/')
                    }}
                  />
                )
              }}
            />
            <Route
              path="/create-klink"
              exact
              render={({ history }) => {
                return (
                  <CreateKlinkDialog
                    defaultData={window.api.getClipboardText()}
                    createKlink={async (klinkRequest) => {
                      const data = await window.api.createKlink(klinkRequest)

                      history.replace(`/klink/${data.token}/copy`)
                    }}
                    close={window.api.close}
                  />
                )
              }}
            />
            <Route
              path="/my-klinks"
              exact
              render={() => <Klinks getKlinks={window.api.getCurrentUserKlinks} />}
            />
            <Route
              path="/deklink"
              exact
              render={() => {
                return (
                  <DeklinkDialog
                    editKlink={window.api.editKlink}
                    readKlink={window.api.readKlink}
                    close={window.api.close}
                  />
                )
              }}
            />
            <Route
              path="/klink/:token/copy"
              exact
              render={({ match }) => {
                const link = window.api.generateKlink(match.params.token)

                return (
                  <CopyKlinkDialog
                    link={link}
                    copyLink={() => window.api.writeClipboardText(link)}
                    close={window.api.close}
                  />
                )
              }}
            />
            <Route
              path="/klink/:token"
              exact
              render={({ match }) => {
                return (
                  <KlinkDetailsDialog
                    token={match.params.token}
                    readKlink={window.api.readKlink}
                    editKlink={async (klinkDetails) => {
                      await window.api.editKlink(match.params.token, klinkDetails)

                      window.api.close()
                    }}
                    close={window.api.close}
                  />
                )
              }}
            />
          </Switch>
        </AppContainer>
      </ThemeProvider>
    </Router>
  )
}

export default App
