import React, { useEffect, useState } from 'react'
import AddBoxIcon from '@material-ui/icons/AddBox'
import DeleteIcon from '@material-ui/icons/DeleteOutline'
import { IconButton } from '@material-ui/core'
import { Contact } from '../../types'
import { ShareWithItem } from './ShareWithItem'
import { getContacts } from '../ContactsRepository'
import { ShareWithHeader, ContactContainer } from './styles'

type Props = {
  values: Contact[],
  onChange(newValue: Contact[]): void,
}

const EMPTY_CONTACT = { phone: '', name: '' }

const ShareWithField = ({ values, onChange }: Props) => {
  const [contacts, setContacts] = useState<Contact[]>([])

  useEffect(() => {
    getContacts()
      .then(array => {
        return {
          data: array
        }
      })
      .then(data => {
        setContacts(data.data)
      })
  }, [])

  const handleInputChange = (contact: Contact | null, index: number) => {
    if (contact) {
      const newValues = [...values]
      newValues[index] = contact
      onChange(newValues)
    }
  }

  return (
    <div>
      <ShareWithHeader>
        <h3>Share with...</h3>
        <IconButton onClick={() => onChange([...values, EMPTY_CONTACT])}>
          <AddBoxIcon color="secondary" />
        </IconButton>
      </ShareWithHeader>
      {values.map((value, index) => {
        return (
          <ContactContainer key={value.phone || index}>
            <ShareWithItem contacts={contacts} value={value} onChange={(contact) => handleInputChange(contact, index)} />
            <IconButton onClick={() => {
              const newValues = [...values]
              newValues.splice(index, 1)
              onChange(newValues)
            }}>
              <DeleteIcon />
            </IconButton>
          </ContactContainer>
        )
      })}
    </div>
  )
}

export default ShareWithField
